@@ -45,7 +45,7 @@ import ppauto
 import ppcommon
 
 copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
-version = "1.6.2"
+version = "1.6.4"
 
 # Reconnect persistent rworkers in seconds.
 RECONNECT_WAIT_TIME = 5
@@ -127,9 +127,7 @@ class _Task(object):
 class _Worker(object):
     """Local worker class
     """
-    command = [sys.executable, "-u",
-            os.path.dirname(os.path.abspath(__file__))
-            + os.sep + "ppworker.py"]
+    command = [sys.executable, "-u", "-m", "ppworker"]
 
     command.append("2>/dev/null")
 
@@ -547,6 +545,7 @@ class Server(object):
                         % (stat.njobs, 100.0*stat.njobs/totaljobs, stat.time,
                         stat.time/stat.njobs, ppserver, )
         print "Time elapsed since server creation", walltime
+        print self.__active_tasks, "active tasks,", self.get_ncpus(), "cores"
 
         if not self.__accurate_stats:
             print "WARNING: statistics provided above is not accurate" \
