@@ -31,13 +31,16 @@ http://www.parallelpython.com - updates, documentation, examples and support
 forums
 """
 
+import atexit
 import logging
+import errno
 import getopt
 import sys
 import socket
 import threading
 import random
 import string
+import signal
 import time
 import os
 
@@ -47,7 +50,7 @@ import ppcommon
 import pptransport
 
 copyright = "Copyright (c) 2005-2012 Vitalii Vanovschi. All rights reserved"
-version = "1.6.2"
+version = "1.6.4"
 
 LISTEN_SOCKET_TIMEOUT = 20
 
@@ -66,9 +69,13 @@ class _NetworkServer(pp.Server):
 
     def __init__(self, ncpus="autodetect", interface="0.0.0.0",
                 broadcast="255.255.255.255", port=None, secret=None,
-                timeout=None, restart=False, proto=2, socket_timeout=3600):
+                timeout=None, restart=False, proto=2, socket_timeout=3600, pid_file=None):
         pp.Server.__init__(self, ncpus, (), secret, restart,
                 proto, socket_timeout)
+        if pid_file:
+          with open(pid_file, 'w') as pfile:
+            print >>pfile, os.getpid()
+          atexit.register(os.remove, pid_file)
         self.host = interface
         self.bcast = broadcast
         if port is not None:
@@ -123,20 +130,26 @@ class _NetworkServer(pp.Server):
         try:
             while 1:
                 csocket = None
-                #accept connections from outside
+                # accept connections from outside
                 try:
                     (csocket, address) = self.ssocket.accept()
                 except socket.timeout:
                     pass
+                # don't exit on an interupt due to a signal
+                except socket.error as e: 
+                    if e.errno == errno.EINTR:
+                      pass
                 if self._exiting:
                     return                
-                #now do something with the clientsocket
-                #in this case, we'll pretend this is a threaded server
+                # now do something with the clientsocket
+                # in this case, we'll pretend this is a threaded server
                 if csocket:
                     ppcommon.start_thread("client_socket",  self.crun, (csocket,  ))
+        except KeyboardInterrupt:
+            pass
         except:
-            if pp.SHOW_EXPECTED_EXCEPTIONS:
-                self.logger.debug("Exception in listen method (possibly expected)", exc_info=True)
+            self.logger.debug("Exception in listen method (possibly expected)", exc_info=True)
+        finally:
             self.logger.debug("Closing server socket")
             self.ssocket.close()            
 
@@ -269,6 +282,11 @@ def parse_config(file_loc):
         args['socket_timeout'] = config['network'].as_int('socket_timeout')
     except:
         pass
+
+    try:
+        args['pid_file'] = config['general'].get('pid_file')
+    except:
+        pass
     # Return a tuple of the args dict and autodiscovery variable
     return args, autodiscovery
 
@@ -279,7 +297,7 @@ def print_usage():
     print "Usage: ppserver.py [-hdar] [-f format] [-n proto]"\
             " [-c config_path] [-i interface] [-b broadcast]"\
             " [-p port] [-w nworkers] [-s secret] [-t seconds]"\
-            " [-k seconds]"
+            " [-k seconds] [-P pid_file]"
     print
     print "Options: "
     print "-h                 : this help message"
@@ -298,7 +316,10 @@ def print_usage():
     print "-t seconds         : timeout to exit if no connections with "\
             "clients exist"
     print "-k seconds         : socket timeout in seconds"
+    print "-P pid_file        : file to write PID to"
     print
+    print "To print server stats send SIGUSR1 to its main process (unix only). "
+    print 
     print "Due to the security concerns always use a non-trivial secret key."
     print "Secret key set by -s switch will override secret key assigned by"
     print "pp_secret variable in .pythonrc.py"
@@ -309,7 +330,7 @@ def print_usage():
 
 def create_network_server(argv):
     try:
-        opts, args = getopt.getopt(argv, "hdarn:c:b:i:p:w:s:t:f:k:", ["help"])
+        opts, args = getopt.getopt(argv, "hdarn:c:b:i:p:w:s:t:f:k:P:", ["help"])
     except getopt.GetoptError:
         print_usage()
         raise
@@ -351,6 +372,8 @@ def create_network_server(argv):
             args["timeout"] = int(arg)
         elif opt == "-k":
             args["socket_timeout"] = int(arg)
+        elif opt == "-P":
+            args["pid_file"] = arg
 
     log_handler = logging.StreamHandler()
     log_handler.setFormatter(logging.Formatter(log_format))
@@ -362,9 +385,14 @@ def create_network_server(argv):
         server.broadcast()
     return server    
     
+def signal_handler(signum, stack):
+    """Prints server stats when SIGUSR1 is received (unix only). """
+    server.print_stats()
 
 if __name__ == "__main__":
     server = create_network_server(sys.argv[1:])
+    if hasattr(signal, "SIGUSR1"):
+      signal.signal(signal.SIGUSR1, signal_handler)
     server.listen()
     #have to destroy it here explicitly otherwise an exception
     #comes out in Python 2.4
